<?php

namespace onespace\tools\activeApi\components\clients\teams\text;

/**
 * Set the horizontal alignment of a text block in a teams message
 *
 * @see https://adaptivecards.io/schemas/adaptive-card.json#HorizontalAlignment
 */
enum TextHorizontalAlignment: string {
    case UNSET = "UNSET";
    case LEFT = "left";
    case CENTER = "center";
    case RIGHT = "right";
}
