<?php

// This class was automatically generated by a giiant build task
// You should not change it manually as it will be overwritten on next build

namespace onespace\tools\activeApi\models\zoho\base;

use yii\helpers\ArrayHelper;

/**
 * This is the base-model class for table "zoho_settings".
 *
 * @property integer $id
 * @property string $setting_group
 * @property string $setting_key
 * @property string $setting_value
 */
abstract class ZohoSettings extends \yii\db\ActiveRecord {
    /**
     * @inheritdoc
     */
    public static function tableName() {
        return 'zoho_settings';
    }

    /**
     * @inheritdoc
     */
    public function rules() {
        $parentRules = parent::rules();
        return ArrayHelper::merge($parentRules, [
            [['setting_key'], 'required'],
            [['setting_group', 'setting_key', 'setting_value'], 'string', 'max' => 254],
        ]);
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels() {
        return ArrayHelper::merge(parent::attributeLabels(), [
            'id' => 'ID',
            'setting_group' => 'Setting Group',
            'setting_key' => 'Setting Key',
            'setting_value' => 'Setting Value',
        ]);
    }

    /**
     * @inheritdoc
     */
    public function attributeHints() {
        return ArrayHelper::merge(parent::attributeHints(), [
            'setting_group' => 'A meta key to group together certain sets of keys.',
            'setting_key' => 'The key identifier of the setting.',
            'setting_value' => 'The setting value.',
        ]);
    }
}
